/**
 *<ATW-APP This APP provides information about the news and washing machines in the ATW-Dormitory>
 *Copyright (C) <2013>  <Mirko Grünenthal & Ingmar Wolf>
 *
 *@author Mirko Grünenthal <mirko.gruenenthal@gmail.com> 
 *@author Ingmar Wolf <ingmar@atw-goettingen.de>
 *
 *This program is free software: you can redistribute it and/or modify
 *it under the terms of the GNU General Public License as published by
 *the Free Software Foundation, either version 3 of the License, or
 *(at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.goe.atw.waschmaschinenlive;


import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.webkit.WebView;
import android.widget.ImageView;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.widget.Toast;


/**
 * Tabs for the news and washing machines, reloads images when shaking
 */

public class InitMainMenu extends Activity implements SensorEventListener {

    private ImageView house12aTabImageView;
    private ImageView house6TabImageView;
    private ImageView house18aTabImageView;
    private ImageView house24gTabImageView;


    private String[] houseArray;
    private int houseCounter = 0;

    private float mLastX, mLastY;
    private boolean mInitialized;
    private SensorManager mSensorManager;
    private Sensor mAccelerometer;
    private long mShakeTimestamp;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


        setContentView(R.layout.tab_menu);

        house12aTabImageView = (ImageView) findViewById(R.id.house12a_tab_image_view);
        house6TabImageView = (ImageView) findViewById(R.id.house6_tab_image_view);
        house18aTabImageView = (ImageView) findViewById(R.id.house18a_tab_image_view);
        house24gTabImageView = (ImageView) findViewById(R.id.house24g_tab_image_view);


        houseArray = getResources().getStringArray(R.array.houseArray);
        loadPicture(0);

        // Tabs
        TabHost tabHost = (TabHost) findViewById(R.id.tabHost);
        tabHost.setup();

        TabSpec newsTabSpec = tabHost.newTabSpec(getString(R.string.news));
        newsTabSpec.setContent(R.id.news_tab_web);
        newsTabSpec.setIndicator(getString(R.string.news));


        TabSpec house12aSpec = tabHost.newTabSpec(getString(R.string.house12a));
        house12aSpec.setContent(R.id.house12a_tab_image_view);
        house12aSpec.setIndicator(getString(R.string.house12a));

        TabSpec house6Spec = tabHost.newTabSpec(getString(R.string.house6));
        house6Spec.setIndicator(getString(R.string.house6));
        house6Spec.setContent(R.id.house6_tab);

        TabSpec house18aSpec = tabHost.newTabSpec(getString(R.string.house18a));
        house18aSpec.setIndicator(getString(R.string.house18a));
        house18aSpec.setContent(R.id.house18a_tab);

        TabSpec house24gSpec = tabHost.newTabSpec(getString(R.string.house24g));
        house24gSpec.setIndicator(getString(R.string.house24g));
        house24gSpec.setContent(R.id.house24g_tab);

        tabHost.addTab(newsTabSpec);
        tabHost.addTab(house12aSpec);
        tabHost.addTab(house6Spec);
        tabHost.addTab(house18aSpec);
        tabHost.addTab(house24gSpec);


        // ShakeListener
        mInitialized = false;
        mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        mAccelerometer = mSensorManager
                .getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        mSensorManager.registerListener(this, mAccelerometer,
                SensorManager.SENSOR_DELAY_NORMAL);
    }


    /**
     * Asynchronous task for fetching data from network
     */
    private class DownloadPictureAsyncTask extends
            AsyncTask<String, Void, Bitmap> {
        @Override
        protected Bitmap doInBackground(String... urls) {

            ImageDownloader imageDownloader;
            imageDownloader = new ImageDownloader();
            return imageDownloader.getBitmapFromUrl(urls[0]);
        }


        @Override
        protected void onPostExecute(Bitmap plotBitmap) {
            if (houseCounter == 0) {
                house12aTabImageView.setImageBitmap(plotBitmap);
                houseCounter = 1;
                loadPicture(houseCounter);
            } else if (houseCounter == 1) {
                house6TabImageView.setImageBitmap(plotBitmap);
                houseCounter = 2;
                loadPicture(houseCounter);
            } else if (houseCounter == 2) {
                house18aTabImageView.setImageBitmap(plotBitmap);
                houseCounter = 3;
                loadPicture(houseCounter);
            } else if (houseCounter == 3) {
                house24gTabImageView.setImageBitmap(plotBitmap);
                houseCounter = 0;

              //WebView with News
                WebView browser = (WebView) findViewById(R.id.news_tab_web);
                browser.loadUrl(getString(R.string.news_url));


                Toast.makeText(getBaseContext(), getString(R.string.refreshed),
                        Toast.LENGTH_SHORT).show();
            }
        }
    }


    /**
     * Calls an asynchronous task {@link DownloadPictureAsyncTask}
     *
     * @param house String array of length 1
     */
    public void loadPicture(int house) {
        DownloadPictureAsyncTask task = new DownloadPictureAsyncTask();
        //checks whether the Task has finished
        if (task.getStatus() == DownloadPictureAsyncTask.Status.FINISHED) {
            task.execute(houseArray[house]);
        } else if (task.getStatus() == DownloadPictureAsyncTask.Status.PENDING) {
            task.execute(houseArray[house]);
        }
    }

    protected void onResume() {
        super.onResume();
        mSensorManager.registerListener(this, mAccelerometer,
                SensorManager.SENSOR_DELAY_NORMAL);
    }

    protected void onPause() {
        super.onPause();
        mSensorManager.unregisterListener(this);
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
        // No function needed
    }

    public void onSensorChanged(SensorEvent event) {
        float x = event.values[0];
        float y = event.values[1];
        if (!mInitialized) {
            mLastX = x;
            mLastY = y;
            mInitialized = true;
        } else {
            float deltaX = Math.abs(mLastX - x);
            float deltaY = Math.abs(mLastY - y);
            float NOISE = (float) 8.0;
            if (deltaX < NOISE)
                deltaX = (float) 0.0;
            if (deltaY < NOISE)
                deltaY = (float) 0.0;
            mLastX = x;
            mLastY = y;

            if (deltaX > 0.0 | deltaY > 0.0) {
                // stops too many refreshes (1000ms), also checks whether die loop has finished,
                //otherwise many shakes will cause a messed up housecount,
                //due to the asynchronous nature of our DownloadPictureAsyncTask, resulting in
                //Bitmaps being loaded into the wrong Tabs)
                final long now = System.currentTimeMillis();
                if (mShakeTimestamp + 1000 > now) {
                    return;
                }
                if (mShakeTimestamp + 1000 < now) {
                    mShakeTimestamp = now;
                    if (houseCounter == 0) {
                        loadPicture(0);
                    }
                }
            }
        }
    }
}

	

